# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/transaction_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import ledger_offset_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2
from . import transaction_filter_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__filter__pb2
from . import transaction_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0com/daml/ledger/api/v1/transaction_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a*com/daml/ledger/api/v1/ledger_offset.proto\x1a/com/daml/ledger/api/v1/transaction_filter.proto\x1a(com/daml/ledger/api/v1/transaction.proto\"\x86\x02\n\x16GetTransactionsRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12:\n\x05\x62\x65gin\x18\x02 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x05\x62\x65gin\x12\x36\n\x03\x65nd\x18\x03 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x03\x65nd\x12\x41\n\x06\x66ilter\x18\x04 \x01(\x0b\x32).com.daml.ledger.api.v1.TransactionFilterR\x06\x66ilter\x12\x18\n\x07verbose\x18\x05 \x01(\x08R\x07verbose\"b\n\x17GetTransactionsResponse\x12G\n\x0ctransactions\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0ctransactions\"j\n\x1bGetTransactionTreesResponse\x12K\n\x0ctransactions\x18\x01 \x03(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0ctransactions\"\x87\x01\n\x1eGetTransactionByEventIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12\x19\n\x08\x65vent_id\x18\x02 \x01(\tR\x07\x65ventId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"\x8e\x01\n\x19GetTransactionByIdRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\x12%\n\x0etransaction_id\x18\x02 \x01(\tR\rtransactionId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\"c\n\x16GetTransactionResponse\x12I\n\x0btransaction\x18\x01 \x01(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTreeR\x0btransaction\"c\n\x1aGetFlatTransactionResponse\x12\x45\n\x0btransaction\x18\x01 \x01(\x0b\x32#.com.daml.ledger.api.v1.TransactionR\x0btransaction\"2\n\x13GetLedgerEndRequest\x12\x1b\n\tledger_id\x18\x01 \x01(\tR\x08ledgerId\"T\n\x14GetLedgerEndResponse\x12<\n\x06offset\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x06offset\"\x1f\n\x1dGetLatestPrunedOffsetsRequest\"\x99\x02\n\x1eGetLatestPrunedOffsetsResponse\x12p\n\"participant_pruned_up_to_inclusive\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\x1eparticipantPrunedUpToInclusive\x12\x84\x01\n-all_divulged_contracts_pruned_up_to_inclusive\x18\x02 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffsetR\'allDivulgedContractsPrunedUpToInclusive2\x87\x08\n\x12TransactionService\x12t\n\x0fGetTransactions\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a/.com.daml.ledger.api.v1.GetTransactionsResponse0\x01\x12|\n\x13GetTransactionTrees\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a\x33.com.daml.ledger.api.v1.GetTransactionTreesResponse0\x01\x12\x81\x01\n\x17GetTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12w\n\x12GetTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12\x89\x01\n\x1bGetFlatTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12\x7f\n\x16GetFlatTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12i\n\x0cGetLedgerEnd\x12+.com.daml.ledger.api.v1.GetLedgerEndRequest\x1a,.com.daml.ledger.api.v1.GetLedgerEndResponse\x12\x87\x01\n\x16GetLatestPrunedOffsets\x12\x35.com.daml.ledger.api.v1.GetLatestPrunedOffsetsRequest\x1a\x36.com.daml.ledger.api.v1.GetLatestPrunedOffsetsResponseB\x96\x01\n\x16\x63om.daml.ledger.api.v1B\x1cTransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.daml.ledger.api.v1.transaction_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\034TransactionServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _globals['_GETTRANSACTIONSREQUEST']._serialized_start=212
  _globals['_GETTRANSACTIONSREQUEST']._serialized_end=474
  _globals['_GETTRANSACTIONSRESPONSE']._serialized_start=476
  _globals['_GETTRANSACTIONSRESPONSE']._serialized_end=574
  _globals['_GETTRANSACTIONTREESRESPONSE']._serialized_start=576
  _globals['_GETTRANSACTIONTREESRESPONSE']._serialized_end=682
  _globals['_GETTRANSACTIONBYEVENTIDREQUEST']._serialized_start=685
  _globals['_GETTRANSACTIONBYEVENTIDREQUEST']._serialized_end=820
  _globals['_GETTRANSACTIONBYIDREQUEST']._serialized_start=823
  _globals['_GETTRANSACTIONBYIDREQUEST']._serialized_end=965
  _globals['_GETTRANSACTIONRESPONSE']._serialized_start=967
  _globals['_GETTRANSACTIONRESPONSE']._serialized_end=1066
  _globals['_GETFLATTRANSACTIONRESPONSE']._serialized_start=1068
  _globals['_GETFLATTRANSACTIONRESPONSE']._serialized_end=1167
  _globals['_GETLEDGERENDREQUEST']._serialized_start=1169
  _globals['_GETLEDGERENDREQUEST']._serialized_end=1219
  _globals['_GETLEDGERENDRESPONSE']._serialized_start=1221
  _globals['_GETLEDGERENDRESPONSE']._serialized_end=1305
  _globals['_GETLATESTPRUNEDOFFSETSREQUEST']._serialized_start=1307
  _globals['_GETLATESTPRUNEDOFFSETSREQUEST']._serialized_end=1338
  _globals['_GETLATESTPRUNEDOFFSETSRESPONSE']._serialized_start=1341
  _globals['_GETLATESTPRUNEDOFFSETSRESPONSE']._serialized_end=1622
  _globals['_TRANSACTIONSERVICE']._serialized_start=1625
  _globals['_TRANSACTIONSERVICE']._serialized_end=2656
# @@protoc_insertion_point(module_scope)
