# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/experimental_features.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2com/daml/ledger/api/v1/experimental_features.proto\x12\x16\x63om.daml.ledger.api.v1\"\xd6\x07\n\x14\x45xperimentalFeatures\x12v\n\x18self_service_error_codes\x18\x01 \x01(\x0b\x32\x39.com.daml.ledger.api.v1.ExperimentalSelfServiceErrorCodesB\x02\x18\x01R\x15selfServiceErrorCodes\x12O\n\x0bstatic_time\x18\x02 \x01(\x0b\x32..com.daml.ledger.api.v1.ExperimentalStaticTimeR\nstaticTime\x12i\n\x15\x63ommand_deduplication\x18\x03 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.CommandDeduplicationFeaturesR\x14\x63ommandDeduplication\x12\x62\n\x12optional_ledger_id\x18\x04 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.ExperimentalOptionalLedgerIdR\x10optionalLedgerId\x12R\n\x0c\x63ontract_ids\x18\x05 \x01(\x0b\x32/.com.daml.ledger.api.v1.ExperimentalContractIdsR\x0b\x63ontractIds\x12\x65\n\x13\x63ommitter_event_log\x18\x06 \x01(\x0b\x32\x35.com.daml.ledger.api.v1.ExperimentalCommitterEventLogR\x11\x63ommitterEventLog\x12g\n\x13\x65xplicit_disclosure\x18\x07 \x01(\x0b\x32\x36.com.daml.ledger.api.v1.ExperimentalExplicitDisclosureR\x12\x65xplicitDisclosure\x12\x9e\x01\n(user_and_party_local_metadata_extensions\x18\x08 \x01(\x0b\x32G.com.daml.ledger.api.v1.ExperimentalUserAndPartyLocalMetadataExtensionsR#userAndPartyLocalMetadataExtensions\x12\x61\n\x14\x61\x63s_active_at_offset\x18\t \x01(\x0b\x32\x30.com.daml.ledger.api.v1.AcsActiveAtOffsetFeatureR\x11\x61\x63sActiveAtOffset\"\'\n!ExperimentalSelfServiceErrorCodes:\x02\x18\x01\"6\n\x16\x45xperimentalStaticTime\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"\xcb\x02\n\x1c\x43ommandDeduplicationFeatures\x12{\n\x1c\x64\x65\x64uplication_period_support\x18\x01 \x01(\x0b\x32\x39.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupportR\x1a\x64\x65\x64uplicationPeriodSupport\x12_\n\x12\x64\x65\x64uplication_type\x18\x02 \x01(\x0e\x32\x30.com.daml.ledger.api.v1.CommandDeduplicationTypeR\x11\x64\x65\x64uplicationType\x12M\n#max_deduplication_duration_enforced\x18\x03 \x01(\x08R maxDeduplicationDurationEnforced\"\x1e\n\x1c\x45xperimentalOptionalLedgerId\"\xbf\x03\n!CommandDeduplicationPeriodSupport\x12n\n\x0eoffset_support\x18\x01 \x01(\x0e\x32G.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupport.OffsetSupportR\roffsetSupport\x12t\n\x10\x64uration_support\x18\x02 \x01(\x0e\x32I.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupport.DurationSupportR\x0f\x64urationSupport\"d\n\rOffsetSupport\x12\x18\n\x14OFFSET_NOT_SUPPORTED\x10\x00\x12\x19\n\x15OFFSET_NATIVE_SUPPORT\x10\x01\x12\x1e\n\x1aOFFSET_CONVERT_TO_DURATION\x10\x02\"N\n\x0f\x44urationSupport\x12\x1b\n\x17\x44URATION_NATIVE_SUPPORT\x10\x00\x12\x1e\n\x1a\x44URATION_CONVERT_TO_OFFSET\x10\x01\"\xa5\x01\n\x17\x45xperimentalContractIds\x12S\n\x02v1\x18\x01 \x01(\x0e\x32\x43.com.daml.ledger.api.v1.ExperimentalContractIds.ContractIdV1SupportR\x02v1\"5\n\x13\x43ontractIdV1Support\x12\x0c\n\x08SUFFIXED\x10\x00\x12\x10\n\x0cNON_SUFFIXED\x10\x01\"\xcd\x01\n\x1d\x45xperimentalCommitterEventLog\x12q\n\x0e\x65vent_log_type\x18\x01 \x01(\x0e\x32K.com.daml.ledger.api.v1.ExperimentalCommitterEventLog.CommitterEventLogTypeR\x0c\x65ventLogType\"9\n\x15\x43ommitterEventLogType\x12\x0f\n\x0b\x43\x45NTRALIZED\x10\x00\x12\x0f\n\x0b\x44ISTRIBUTED\x10\x01\">\n\x1e\x45xperimentalExplicitDisclosure\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"O\n/ExperimentalUserAndPartyLocalMetadataExtensions\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"8\n\x18\x41\x63sActiveAtOffsetFeature\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported*I\n\x18\x43ommandDeduplicationType\x12\x0e\n\nASYNC_ONLY\x10\x00\x12\x1d\n\x19\x41SYNC_AND_CONCURRENT_SYNC\x10\x01\x42\x98\x01\n\x16\x63om.daml.ledger.api.v1B\x1e\x45xperimentalFeaturesOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.daml.ledger.api.v1.experimental_features_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\036ExperimentalFeaturesOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _EXPERIMENTALFEATURES.fields_by_name['self_service_error_codes']._options = None
  _EXPERIMENTALFEATURES.fields_by_name['self_service_error_codes']._serialized_options = b'\030\001'
  _EXPERIMENTALSELFSERVICEERRORCODES._options = None
  _EXPERIMENTALSELFSERVICEERRORCODES._serialized_options = b'\030\001'
  _globals['_COMMANDDEDUPLICATIONTYPE']._serialized_start=2555
  _globals['_COMMANDDEDUPLICATIONTYPE']._serialized_end=2628
  _globals['_EXPERIMENTALFEATURES']._serialized_start=79
  _globals['_EXPERIMENTALFEATURES']._serialized_end=1061
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._serialized_start=1063
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._serialized_end=1102
  _globals['_EXPERIMENTALSTATICTIME']._serialized_start=1104
  _globals['_EXPERIMENTALSTATICTIME']._serialized_end=1158
  _globals['_COMMANDDEDUPLICATIONFEATURES']._serialized_start=1161
  _globals['_COMMANDDEDUPLICATIONFEATURES']._serialized_end=1492
  _globals['_EXPERIMENTALOPTIONALLEDGERID']._serialized_start=1494
  _globals['_EXPERIMENTALOPTIONALLEDGERID']._serialized_end=1524
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT']._serialized_start=1527
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT']._serialized_end=1974
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_OFFSETSUPPORT']._serialized_start=1794
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_OFFSETSUPPORT']._serialized_end=1894
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_DURATIONSUPPORT']._serialized_start=1896
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_DURATIONSUPPORT']._serialized_end=1974
  _globals['_EXPERIMENTALCONTRACTIDS']._serialized_start=1977
  _globals['_EXPERIMENTALCONTRACTIDS']._serialized_end=2142
  _globals['_EXPERIMENTALCONTRACTIDS_CONTRACTIDV1SUPPORT']._serialized_start=2089
  _globals['_EXPERIMENTALCONTRACTIDS_CONTRACTIDV1SUPPORT']._serialized_end=2142
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG']._serialized_start=2145
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG']._serialized_end=2350
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG_COMMITTEREVENTLOGTYPE']._serialized_start=2293
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG_COMMITTEREVENTLOGTYPE']._serialized_end=2350
  _globals['_EXPERIMENTALEXPLICITDISCLOSURE']._serialized_start=2352
  _globals['_EXPERIMENTALEXPLICITDISCLOSURE']._serialized_end=2414
  _globals['_EXPERIMENTALUSERANDPARTYLOCALMETADATAEXTENSIONS']._serialized_start=2416
  _globals['_EXPERIMENTALUSERANDPARTYLOCALMETADATAEXTENSIONS']._serialized_end=2495
  _globals['_ACSACTIVEATOFFSETFEATURE']._serialized_start=2497
  _globals['_ACSACTIVEATOFFSETFEATURE']._serialized_end=2553
# @@protoc_insertion_point(module_scope)
